local Talented = Talented
local ipairs = ipairs

local LAYOUT_BASE_X = 4
local LAYOUT_BASE_Y = 24
-- local LAYOUT_MAX_COLUMNS = 4
-- local LAYOUT_MAX_ROWS = 9

local LAYOUT_OFFSET_X, LAYOUT_OFFSET_Y, LAYOUT_DELTA_X, LAYOUT_DELTA_Y
local LAYOUT_SIZE_X, LAYOUT_SIZE_Y

local function RecalcLayout(offset)
	if LAYOUT_OFFSET_X ~= offset then

		LAYOUT_OFFSET_X = offset
		LAYOUT_OFFSET_Y = LAYOUT_OFFSET_X

		LAYOUT_DELTA_X = LAYOUT_OFFSET_X / 2
		LAYOUT_DELTA_Y = LAYOUT_OFFSET_Y / 2

		LAYOUT_SIZE_X = --[[LAYOUT_MAX_COLUMNS]] 4 * LAYOUT_OFFSET_X + LAYOUT_DELTA_X
		LAYOUT_SIZE_Y = --[[LAYOUT_MAX_ROWS]] 11 * LAYOUT_OFFSET_Y + LAYOUT_DELTA_Y

		return true
	end
end

local function offset(row, column)
	return (column - 1) * LAYOUT_OFFSET_X + LAYOUT_DELTA_X, - ((row - 1) * LAYOUT_OFFSET_Y + LAYOUT_DELTA_Y)
end

Talented.elements = {}
function Talented:SetUIElement(element, ...)
	self.elements[strjoin("-", ...)] = element
end
function Talented:GetUIElement(...)
	return self.elements[strjoin("-", ...)]
end

function Talented:CreateTalentMatrix(class, force)
	if not force and class ~= nil and self.viewclass == class then return true end
	local talents = self:UncompressSpellData(class)

	local parent = self.base

	self.Pool:changeSet("player")

	if not LAYOUT_OFFSET_X then
		RecalcLayout(self.db.profile.offset)
	end

	for tab, tree in ipairs(talents) do
		local frame = self:MakeTalentFrame(parent, LAYOUT_SIZE_X, LAYOUT_SIZE_Y)
		frame.tab = tab

		local background = self.tabdata[class][tab].background
		frame.topleft:SetTexture("Interface\\TalentFrame\\"..background.."-TopLeft")
		frame.topright:SetTexture("Interface\\TalentFrame\\"..background.."-TopRight")
		frame.bottomleft:SetTexture("Interface\\TalentFrame\\"..background.."-BottomLeft")
		frame.bottomright:SetTexture("Interface\\TalentFrame\\"..background.."-BottomRight")

		self:SetUIElement(frame, tab)

		for idx, talent in ipairs(tree) do
			local button = self:MakeButton(frame)
			button.tab = tab
			button.id = idx

			self:SetUIElement(button, tab, idx)

			button:SetPoint("TOPLEFT", offset(talent.row, talent.column))
			button.texture:SetTexture(self:GetTalentIcon(class, tab, idx))
			button:Show()
		end

		for idx, talent in ipairs(tree) do
			local req = talent.req
			if req then
				local elements = {}
				self.DrawLine(elements, frame, offset, talent.row, talent.column, tree[req].row, tree[req].column)
				self:SetUIElement(elements, tab, idx, req)
			end
		end

		frame:SetPoint("TOPLEFT", (tab-1) * LAYOUT_SIZE_X + LAYOUT_BASE_X, -LAYOUT_BASE_Y)
	end
	parent:SetWidth(#talents * LAYOUT_SIZE_X + LAYOUT_BASE_X * 2)
	parent:SetHeight(LAYOUT_SIZE_Y + LAYOUT_BASE_Y + LAYOUT_BASE_X)
	parent:SetPoint(self.db.profile.point, UIParent, self.db.profile.point, self.db.profile.offsetx, self.db.profile.offsety)

	parent:SetScale(self.db.profile.scale)

	self.viewclass = class
	return true
end

function Talented:SaveFramePosition()
	local p, _ = self.db.profile
	p.point, _, _, p.offsetx, p.offsety = self.base:GetPoint()
	self.base:SetPoint(p.point, UIParent, p.point, p.offsetx, p.offsety)
end

function Talented:ReLayout()
	if RecalcLayout(self.db.profile.offset) then
		self:CreateTalentMatrix(self.viewclass, true)
	end
	self.base:SetScale(self.db.profile.scale)
end


